/** ###################################################################
**     THIS BEAN MODULE IS GENERATED BY THE TOOL. DO NOT MODIFY IT.
**     Filename  : SPI_FS.C
**     Project   : DC17
**     Processor : MC56F8006_32_LQFP
**     Beantype  : SWSPI
**     Version   : Bean 01.018, Driver 01.15, CPU db: 3.00.169
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 8/2/2009, 7:53 PM
**     Abstract  :
**          This bean implements an external SPI communication interface specialy 
**          designed for SPI compatible beans. The interface uses three general-purpose 
**          I/O pins, first for the Input line, second for the Output line, third for 
**          the Serial Clock line. Is supported only Master mode with MSB first 
**          communication. 
**     Contents  :
**         RecvChar     - byte SPI_FS_RecvChar(byte *Chr);
**         SendChar     - byte SPI_FS_SendChar(byte Chr);
**         CharsInRxBuf - byte SPI_FS_CharsInRxBuf(word *Chr);
**         CharsInTxBuf - byte SPI_FS_CharsInTxBuf(word *Chr);
**
**     (c) Copyright UNIS, a.s. 1997-2008
**     UNIS, a.s.
**     Jundrovska 33
**     624 00 Brno
**     Czech Republic
**     http      : www.processorexpert.com
**     mail      : info@processorexpert.com
** ###################################################################*/


/* MODULE SPI_FS. */

#include "SPI_FS.h"


#define OVERRUN_ERR  1                 /* Overrun error flag bit   */
#define CHAR_IN_RX   8                 /* Char is in RX buffer     */

static byte CLKshift;
static byte CLKsampl;
static byte InputBuffer;
static byte SerFlag;                   /* Flags for serial communication */
                                       /* Bits: 0 - OverRun error */
                                       /*       1 - Unused */
                                       /*       2 - Unused */
                                       /*       3 - Char in RX buffer */
                                       /*       4 - Unused */
                                       /*       5 - Unused */
                                       /*       6 - Unused */
                                       /*       7 - Unused */

/*
** ===================================================================
**     Method      :  SPI_FS_RecvChar (bean SWSPI)
**
**     Description :
**         If any data received, this method returns one character,
**         otherwise it returns error code (it does not wait for
**         data).
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Chr             - Pointer to received character.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_RXEMPTY - No data in receiver
**                           ERR_OVERRUN - Overrun error is detected
** ===================================================================
*/
byte SPI_FS_RecvChar(byte *Chr)
{
  if(!(SerFlag & CHAR_IN_RX)) {        /* Is char. received */
    return ERR_RXEMPTY;
  }
  *Chr = InputBuffer;                  /* Store the character */
  if(SerFlag & OVERRUN_ERR) {          /* Is "overrun" occured? */
    SerFlag &= ~(OVERRUN_ERR|CHAR_IN_RX); /* If yes, clear flags */
    return ERR_OVERRUN;                /* ... and return error */
  }
  else {
    SerFlag &= ~CHAR_IN_RX;            /* If no, clear flag */
    return ERR_OK;                     /* ... and return */
  }
}

/*
** ===================================================================
**     Method      :  SPI_FS_SendChar (bean SWSPI)
**
**     Description :
**         Send one character to the channel.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Chr             - Character to send.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
** ===================================================================
*/
byte SPI_FS_SendChar(byte Chr)
{
  byte i;
  InputBuffer = 0;
  for(i=0; i<8; i++) {
    Inhr5_PutVal(CLKshift);            /* Set CLK to shift value */
    Inhr4_PutVal((bool)(Chr & 128));   /* Set value on MOSI */
    InputBuffer <<= 1;
    Inhr5_PutVal(CLKsampl);            /* Set CLK to sample value */
    InputBuffer += Inhr3_GetVal()?1:0; /* Read value from MISO */
    Chr <<= 1;
  }
  Inhr4_PutVal(1);                     /* Set value on MOSI */
  if(SerFlag & CHAR_IN_RX) {           /* Is char. received? */
    SerFlag |= OVERRUN_ERR;            /* If yes then set "overrun" flag */
  }
  else {
    SerFlag |= CHAR_IN_RX;             /* If no then set "char in Rx" flag */
  }
  return ERR_OK;
}

/*
** ===================================================================
**     Method      :  SPI_FS_CharsInRxBuf (bean SWSPI)
**
**     Description :
**         Return number of characters in the input buffer.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Chr             - Pointer to number of characters in the
**                           input buffer.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
** ===================================================================
*/
byte SPI_FS_CharsInRxBuf(word *Chr)
{
  *Chr = (word)((SerFlag & CHAR_IN_RX)?(word)1:(word)0); /* Return number of chars in receive buffer */
  return ERR_OK;
}

/*
** ===================================================================
**     Method      :  SPI_FS_CharsInTxBuf (bean SWSPI)
**
**     Description :
**         Return number of characters in the output buffer.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Chr             - Pointer to number of characters in the
**                           output buffer.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
** ===================================================================
*/
byte SPI_FS_CharsInTxBuf(word *Chr)
{
  *Chr = (word)0;                      /* Return number of chars in transmit buffer */
  return ERR_OK;
}

/*
** ===================================================================
**     Method      :  SPI_FS_Init (bean SWSPI)
**
**     Description :
**         Initializes the associated peripheral(s) and the beans 
**         internal variables. The method is called automatically as a 
**         part of the application initialization code.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void SPI_FS_Init(void)
{
  CLKsampl = 1;                        /* Set sample value to 1 */
  CLKshift = 0;                        /* Set shift value to 0 */
  Inhr5_PutVal((bool)1);               /* Set CLK to idle value */
  Inhr4_PutVal((bool)1);               /* Set value on MOSI */
  SerFlag = 0;                         /* Clear flags */
}


/* END SPI_FS. */

/*
** ###################################################################
**
**     This file was created by UNIS Processor Expert 2.99 [04.17]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/

