/******************************************************************************
*
* DEFCON 17 BADGE
*
* Filename:		  DC17_Badge.c
* Author:		    Joe Grand (Kingpin)
* Revision:		  1.0
* Last Updated: April 27, 2009
*
* Description:	Main File for the DEFCON 17 Badge (Freescale MC56F8006)
* Notes:
*
* See DC17_Badge.h for more inf0z...		
*******************************************************************************/

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "PWMRed.h"
#include "PWMGreen.h"
#include "PWMBlue.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "DC17_Badge.h"
#include "SPI_FS.h"
#include "SPI_FS_CS.h"

/****************************************************************************
 ************************** Global variables ********************************
 ***************************************************************************/

// Badge 
badge_state_type badge_state = I2C; // begin in I2C (multi-badge) mode on power-up
badge_type_struct badge_type;
struct packed_flags flags;

// Microphone
unsigned int freqcount = 0;
word	micval = 0;
word  micval_offset;

// A/D
unsigned int AdcBuffer[ADC_MAX_BUFFER_SIZE];
unsigned int AdcWriteIndex = 0;
unsigned int AdcReadIndex = 0;

// LED
led_state_type LEDstate = RED;
unsigned int LEDptr = 0;

// RNG
//unsigned int randtime = 0;
//int randval; 

// Sleep
byte threshold_failure;
word threshold_count;

// Modes
unsigned int soscount = 0;
unsigned int dancetmr = 0; 


/****************************************************************************
 ************************** Functions ***************************************
 ***************************************************************************/
void fs_GetVersion(void);

void dc17_badge(void)
{ 
    byte cmd = (byte) 'V';
	
	dc17_init();
	MICOUT_Disable();
	
	Term1_SendStr("BBBS Initializing");
	Term1_CRLF();
	Term1_SendStr("Talking to microSD card port");
	fs_GetVersion();
	
    Term1_SendStr("Done!");

	while(1) {
	}
}

/*
 device_type 03(hex) : This byte indicates the device type is GOLDELOX-DOS
 silicon_rev This byte indicates the GOLDELOX silicon revision
 pmmc_rev This byte indicates the PmmC firmware revision
 reserved1 This byte is reserved for future support. If the value is 0 then ignore it
 reserved2 This byte is reserved for future support. If the value is 0 then ignore it

\Description This command requests all the necessary information from the device about its
            characteristics and capability.
*/
void fs_GetVersion() {
    byte buf[5];
    unsigned int num;
    
    byte code = 0x56;
    byte cmd = (byte) 'V';
    
    Term1_SendStr("Sending version command");
    //fs_SendChar(code);
    //fs_SendChar(cmd);
    /*
    while (fs_GetCharsInRxBuf() < 5) {
        //Cpu_Delay100US(1);
        delay_ms(1000);
        Term1_SendStr("Waiting for 5 bytes");
        Term1_CRLF();
    }
    Term1_SendStr("5 bytes in the rx buffer");
    
    //Response device_type, silicon_rev, pmmc_rev, reserved1, reserved2
    fs_RecvBlock(buf, 5, &num);
    
    Term1_SendStr("device_type: ");
    Term1_SendNum(buf[0]);
    Term1_CRLF();

    Term1_SendStr("silicon_rev: ");
    Term1_SendNum(buf[1]);
    Term1_CRLF();

    Term1_SendStr("pmmc_rev: ");
    Term1_SendNum(buf[2]);
    Term1_CRLF();
    */
}


void startup_animation(void) {
    int led_bits;
    int count = 10;

    for (; count > 0; count--) {
        for (led_bits = 0; led_bits < 8; led_bits++) {
            PWMRed_SetRatio16((led_bits & 0x01) == 1 ? 0xFFFF : 0);
            PWMGreen_SetRatio16((led_bits & 0x02) == 1 ? 0xFFFF : 0);
            PWMBlue_SetRatio16((led_bits & 0x04) == 1 ? 0xFFFF : 0);
            delay_ms(60);
        }
    }
}

// PIT - used to wake up from sleep mode
void dc17_pit_isr(void) {  	   	

}

void sleep(void) {
    unsigned long   Temp1, Temp2;    

	Term1_SendStr("Sleeping...");
	Term1_CRLF();

    // disable peripherals
   	PWMRed_Disable();
	PWMGreen_Disable();
	PWMBlue_Disable();
	MICOUT_Disable();

	clrRegBit(GPIO_A_PER, PE0);
	clrRegBit(GPIO_A_PER, PE1);
    clrRegBit(GPIO_A_PER, PE2);
				
	// save original PIT timing values
	Temp1 = getReg(PIT_MOD);
	Temp2 = getReg(PIT_CTRL);
			
	// set PIT to interrupt in 1 second to check for sound
	setReg(PIT_MOD, 62500); 
	setRegBitGroup(PIT_CTRL, PRESCALER, 6); 

	Cpu_SetStopMode(); // Zzzzz... (wait for interrupt)
						
    // restore original PIT timing values	    
    setReg(PIT_MOD, Temp1);
    setReg(PIT_CTRL, Temp2);
	    
	threshold_count = 0;
	threshold_failure = 0;
 	PWMRed_Enable(); 		
	PWMGreen_Enable();
	PWMBlue_Enable();	 			
	MICOUT_Enable();

	Term1_SendStr("Woke up!");
	Term1_CRLF();

    // Cpu_SetWaitMode();  // snooze in reduced-power wait mode until the next interrupt
}


/**************************************************************/


void dc17_sleep(void) // sleep, if required
{
    unsigned long   Temp1, Temp2;    

    // disable peripherals
   	PWMRed_Disable();
	PWMGreen_Disable();
	PWMBlue_Disable();
	MICOUT_Disable();
	clrRegBit(GPIO_A_PER, PE0);
	clrRegBit(GPIO_A_PER, PE1);
    clrRegBit(GPIO_A_PER, PE2);

	// save original PIT timing values
	Temp1 = getReg(PIT_MOD);
	Temp2 = getReg(PIT_CTRL);
			
	// set PIT to interrupt in 1 second to check for sound
	setReg(PIT_MOD, 62500); 
	setRegBitGroup(PIT_CTRL, PRESCALER, 6); 

	Cpu_SetStopMode(); // Zzzzz...
						
    // restore original PIT timing values	    
    setReg(PIT_MOD, Temp1);
    setReg(PIT_CTRL, Temp2);
	    
	threshold_count = 0;
	threshold_failure = 0;
 	PWMRed_Enable(); 		
	PWMGreen_Enable();
	PWMBlue_Enable();	 			
	MICOUT_Enable();

	Cpu_SetWaitMode();  // snooze in reduced-power wait mode until the next interrupt
}

/**************************************************************/




/**************************************************************/

void dc17_init(void) // badge start-up/initialization
{
	Cpu_DisableInt(); // disable global interrupts 	
	PWMRed_Enable();  // while we enable all the modules
	PWMGreen_Enable();
	PWMBlue_Enable();
	MICOUT_Enable();

  // take a listen at the current ambient noise
	/*MICOUT_Measure(TRUE); // number of conversions specified in ADC bean, wait for conversion to complete
	MICOUT_GetValue16(&micval); // get a/d sample  
	srand((unsigned int)micval); // ...seed the PRNG with the value */
  		
	Cpu_EnableInt();  // re-enable global interrupts when we're done and ready for action
 		
	// initialize global flags
	flags.PerformFFT = FALSE;
	flags.ptr_update = 0;
	flags.sleep = 0;
	  
  threshold_failure = 0; // counters for auto-power-off
  threshold_count = 0;
}



/**************************************************************
/* INTERRUPT SERVICE ROUTINES
/**************************************************************/

void dc17_t1_isr(void) // TMR1: RGB multiplexing, every 1ms
{
		switch (LEDstate)
		{
				case RED:
					setRegBit(GPIO_A_PER, PE0);
  	  		clrRegBit(GPIO_A_PER, PE1);
  	  		clrRegBit(GPIO_A_PER, PE2);
  				LEDstate = GREEN;
					break;
				case GREEN:
					clrRegBit(GPIO_A_PER, PE0);
  	  		setRegBit(GPIO_A_PER, PE1);
  	  		clrRegBit(GPIO_A_PER, PE2);
  				LEDstate = BLUE;					
  				break;
				case BLUE:
					clrRegBit(GPIO_A_PER, PE0);
  	  		clrRegBit(GPIO_A_PER, PE1);
  	  		setRegBit(GPIO_A_PER, PE2);
  				LEDstate = RED;				
					break;
		}
}

/**************************************************************
*	Function: 	morse
*	Parameters: 16-bit delay value
*	Return:		none
*
*	Transmit a string in Morse code
*
* This function transmits the string pointed to by the cp 
* argument in Morse code via the RGB LED. 
* 
* The transmission speed is set by the UNIT_TIME constant. 
* 
* A static table translates from ASCII to Morse code. Each 
* entry is an unsigned integer, where a zero represents a dot 
* and a one represents a dash. No more than 14 bits may be used.
* Setting bit 15 produces a word space, regardless of any other 
* pattern.
* 
* The Morse code pattern is taken from bit 0, and is shifted 
* right each time an element is sent. A special "marker bit" 
* follows the complete Morse pattern. This marker bit is tested 
* before transmitting each bit; if there are no 1's in bits 
* 1..15, the complete character has been sent.
* 
* For example, an "L" (dit-dah-dit-dit) would be:
* 0000000000010010, with bit zero containing the first dot, 
* bit one the dash, etc.  The marker bit is in bit 4.
*
* Based on code written by Michael M. Dodd, N4CF
*
**************************************************************/

void morse(char *cp)
{  
	unsigned int c;

	TIPIT_Disable(); 	// disable PIT to prevent blending
  	
  PWMRed_SetRatio16(OFF);
  PWMGreen_SetRatio16(OFF);
  PWMBlue_SetRatio16(OFF);
  	
  while ((c = *cp++) != '\0')	 // transmit string
  {  
  	//c = (unsigned int)toupper((int)c); // convert all text to uppercase - there are no lower-case Morse characters
    c -= ' ';                  // adjust for zero-based table

    if (c < 0 || c > 58)       // if character out of range, ignore it
    	continue;

    c = codes[c];              // look up Morse pattern from table

    if (c & SPACE_MASK)        // if the space bit is set...
    {    
    	word_space(); 					 //...then send a word space and continue
      continue;
    }

    while (c & BIT_MASK)       // transmit a character...
    {
    	// ...one bit at a time
     	if (c & 1)
       	send_dash();
      else  
       	send_dot();

      c >>= 1;
    }

    ltr_space();               // send a letter space

  } 
  
  TIPIT_Enable(); 	// re-enable
}

/*
** These functions transmit a dot, a dash, a letter space, and a
** word space.
** 
** Note that a single unit space is automatically transmitted after
** each dot or dash, so the ltr_space() function produces only a
** two-unit pause.
** 
** Also, the word_space() function produces only a four-unit pause
** because the three-unit letter space has already occurred following
** the previous letter.
*/

void send_dot(void)				// dot and space
{
  	PWMRed_SetRatio16(ON);
   	PWMGreen_SetRatio16(ON);
  	PWMBlue_SetRatio16(ON);
  
    delay_ms(UNIT_TIME);
    
  	PWMRed_SetRatio16(OFF);
   	PWMGreen_SetRatio16(OFF);
  	PWMBlue_SetRatio16(OFF);
  
    delay_ms(UNIT_TIME);
}

void send_dash(void)      // dash and space
{
  	PWMRed_SetRatio16(ON);
   	PWMGreen_SetRatio16(ON);
  	PWMBlue_SetRatio16(ON);
  	
  	delay_ms(UNIT_TIME * 3);
    
  	PWMRed_SetRatio16(OFF);
   	PWMGreen_SetRatio16(OFF);
  	PWMBlue_SetRatio16(OFF);
  	
  	delay_ms(UNIT_TIME);
}

void ltr_space(void)      // letter space
{
      delay_ms(UNIT_TIME * 3);
}

void word_space(void)     // word space
{
      delay_ms(UNIT_TIME * 7);
}


/**************************************************************
*	Function: 	delay
*	Parameters: 16-bit delay value
*	Return:		none
*
*	Simple delay loops
* These are not very accurate because they could get interrupted
**************************************************************/
 
void delay_ms(unsigned int ms)  // delay the specified number of milliseconds
{
  unsigned int i;

  for (i = 0; i < ms; ++i)
  	Cpu_Delay100US(10);
}

/****************************** END OF FILE **********************************/

