/*******************************************************************************
   Copyright 2008, 2009 Wearable Inc.; An Illinois, United States Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*******************************************************************************/
/*******************************************************************************
   Name:        DOSonCHIP(TM) Physical Communications Port Definitions
   File Name:   DOSport_UART.h
   Description: UART Communications Definitions (CPU Specific)
*******************************************************************************/

#ifndef DOSPORT_UART_H
#define DOSPORT_UART_H

/* UART DEFINITIONS SPECIFIC TO HOST CPU */
#include "c8051f340.h"                       /* CPU specific definitions */
#define TI1      (SCON1 & 0x02)              /* UART flag that previous byte has been transmitted */
#define TI1_CLR  SCON1 &= ~0x02              /* UART clear flag */
#define TI1_SET  SCON1 |= 0x02               /* UART set flag */
#define RI1      (SCON1 & 0x01)              /* UART flag that byte has been received by */
#define RI1_CLR  SCON1 &= ~0x01              /* UART clear flag */
sbit  DOS_UART_RTS_BUSY = P2^7;              /* P2.7 = CTS0 (open-drain input) */
sbit  DOS_UART_CTS      = P2^6;              /* P2.6 = RTSO (push-pull output) */

#define DOS_PORT_ENABLE                      /* stub */
#define DOS_PORT_DISABLE                     /* stub */

/* FUNCTION PROTOTYPES */
unsigned char DOSportGetNotBusy (void);      /* Poll DOSonCHIP for valid response byte */
unsigned char DOSportPutByte (unsigned char txByte); /* Transmit byte to DOSonCHIP */
unsigned char DOSportGetByte (void);         /* Receive byte from DOSonCHIP */
void          DOSportFlushQ (void);          /* Wait until transmit and/or receive queue is empty */
void          DOSportInit (void);            /* Initialize host port */

#endif
