/*******************************************************************************
   Copyright 2008, 2009 Wearable Inc.; An Illinois, United States Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*******************************************************************************/
/*******************************************************************************
   Name:        DOSonCHIP(TM) Physical Communications Port Functions (SPI)
   File Name:   DOSport_SPI_BitBang.c
   Description: SPI Bit-Bang Communications Subroutines (CPU Specific)
*******************************************************************************/

#ifdef DOS_PORT_SPI_BITBANG

/* FILE INCLUDES */
#include "c8051F340.h"                             /* CPU specific definitions */
#include "DOSport_SPI_BitBang.h"                   /* complimentary include file */
#include "DOSprotocol.h"                           /* DOSonCHIP communication protocol include file */

/******************************************************************************/
unsigned char DOSportGetNotBusy (void)             /* Poll DOSonCHIP for valid response byte */
{
   unsigned long int DOSflagTimeOut = 0xFFFFFFFF;
   unsigned char     response;

   /* return non DOS_HANDSHAKE_WAIT byte or DOS_HANDSHAKE_WAIT byte if timeout */
   while (((response = DOSportGetByte ()) == DOS_HANDSHAKE_WAIT) && DOSflagTimeOut)
   {
      /* Skip over fixed amount of DOS_HANDSHAKE_WAIT bytes */
      /* Wait loop time is CPU specific! */
      /* If available, use a timer interrupt to set DOSflagTimeOut */
      DOSflagTimeOut -= 1;
   }
   return (response);
}

/*******************************************************************************/
unsigned char DOSportPutByte (unsigned char txByte) /* Transmit byte to DOSonCHIP */
/* SPI port simultaneously transmits and receives a byte */
{
   unsigned char i = 8;                            /* 8 bits to a byte */

   while (i > 0)                                   /* repeat for all bits of transmit byte */
   {
      SPIB_MOSI = txByte & 0x80;                   /* output bit */
      txByte    = txByte << 1;                     /* shift transmit byte */
      SPIB_SCK  = 1;                               /* output clock high */
      txByte   |= SPIB_MISO;                       /* input bit */
      /* THERE MUST BE ENOUGH DELAY BETWEEN SPIB_SCK
         CHANGES TO NOT GO BELOW DOSONCHIP REQUIRED PERIOD/2. */
      SPIB_SCK  = 0;                               /* output clock low */
      /* THERE MUST BE ENOUGH DELAY BETWEEN SPIB_SCK
         CHANGES TO NOT GO BELOW DOSONCHIP REQUIRED PERIOD/2. */

      i--;
   }
   return (txByte);                                /* return received byte */
}

/*******************************************************************************/
unsigned char DOSportGetByte (void)                /* Receive byte from DOSonCHIP */
{  
   return (DOSportPutByte (DOS_HANDSHAKE_POLL));   /* transmit polling byte & return received byte */
}

/*******************************************************************************/
void DOSportFlushQ (void)                          /* Wait until transmit/receive queue is empty */
{
}

/*******************************************************************************/
void DOSportInit (void)                            /* Initialize host port */
/* SPI port attributes:                */
/* Data 8-bits sent/received MSB first */
/* Clock polarity = 0                  */
/* Clock phase = 0                     */
{
   unsigned char temp = DOS_HANDSHAKE_OFF;

   /* Initialize SPI BitBang Port */
   SPIB_SCK  = 0;                                  /* Initialize clock polarity */
   SPIB_MOSI = 0;

   /* Wait for DOSonCHIP power-up */
   DOS_PORT_ENABLE;                                /* enable DOSonCHIP communications port */

   while (temp == DOS_HANDSHAKE_OFF)               /* poll until DOSonCHIP is ready to communicate */
   {
     temp = DOSportGetByte();
   }

   DOSportPutByte (DOS_HANDSHAKE_GO);              /* signal to DOSonCHIP that host is ready */

   DOS_PORT_DISABLE;                               /* disable DOSonCHIP communications port */
}

#endif /* DOS_PORT_SPI_BITBANG */
