/*******************************************************************************
   Copyright 2008, 2009 Wearable Inc.; An Illinois, United States Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*******************************************************************************/
/*******************************************************************************
   Name:        DOSonCHIP(TM) Physical Communications Port Definitions
   File Name:   DOSport_SPI_BitBang.h
   Description: SPI Bit-Bang Communications Definitions (CPU Specific)
*******************************************************************************/

#ifndef DOSPORT_SPI_BITBANG_H
#define DOSPORT_SPI_BITBANG_H

/* SPI PINS SPECIFIC TO HOST CPU */
#include "c8051f340.h"                       /* CPU specific definitions */
sbit  SPIB_SCK  = P0^0;                      /* P0.0 = SCK  (push-pull output) */
sbit  SPIB_MISO = P0^1;                      /* P0.1 = MISO (open-drain input) */
sbit  SPIB_MOSI = P0^2;                      /* P0.2 = MOSI (push-pull output) */
sbit  SPIB_NSS  = P0^3;          b            /* P0.3 = NSS  (push-pull output) */

#define DOS_PORT_ENABLE   SPIB_NSS = 0       /* enable DOSonCHIP SPI port */
#define DOS_PORT_DISABLE  SPIB_NSS = 1       /* disable DOSonCHIP SPI port */

/* FUNCTION PROTOTYPES */
unsigned char DOSportGetNotBusy (void);      /* Poll DOSonCHIP for valid response byte */
unsigned char DOSportPutByte (unsigned char txByte); /* Transmit byte to DOSonCHIP */
unsigned char DOSportGetByte (void);         /* Receive byte from DOSonCHIP */
void          DOSportFlushQ (void);          /* Wait until transmit and/or receive queue is empty */
void          DOSportInit (void);            /* Initialize host port */

#endif
