/*******************************************************************************
   Copyright 2008, 2009 Wearable Inc.; An Illinois, United States Corporation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*******************************************************************************/
/*******************************************************************************
   Name:        DOSonCHIP(TM) Physical Communications Port Definitions (UART)
   File Name:   DOSport_UART.c
   Description: UART Communications Subroutines (CPU Specific)
*******************************************************************************/

#ifdef DOS_PORT_UART

/* FILE INCLUDES */
#include "c8051F340.h"                             /* CPU specific definitions */
#include "DOSport_UART.h"                          /* complimentary include file */
#include "DOSprotocol.h"                           /* DOSonCHIP communication protocol include file */

/******************************************************************************/
unsigned char DOSportGetNotBusy (void)             /* Wait for valid response byte */
{
   unsigned char response;

   /* skip DOS_HANDSHAKE_WAIT bytes */
   DOSportPutByte (DOS_HANDSHAKE_POLL);            /* Wake up DOSonCHIP */
   while ((response = DOSportGetByte ()) == DOS_HANDSHAKE_WAIT);
   return response;
}

/*******************************************************************************/
unsigned char DOSportPutByte (unsigned char txByte) /* Transmit byte to DOSonCHIP */
{
   while (!TI1);                                   /* wait for previous byte to be transmitted */
   TI1_CLR;                                        /* manually clear transmit flag */
   /* while (DOS_UART_RTS_BUSY); */                /* wait for DOSonCHIP to be ready to accept bytes */
   SBUF1 = txByte;                                 /* transmit byte to host */

   return 0;
}

/*******************************************************************************/
unsigned char DOSportGetByte (void)                /* Receive byte from DOSonCHIP */
{
   unsigned long int DOSflagTimeOut = 0xFFFFFFFF;

   /* DOS_UART_CTS = 0; */                         /* allow DOSonCHIP to send bytes to host */
   while (!RI1 && DOSflagTimeOut)                  /* wait for byte to be received */
   {
      /* Wait loop time is CPU specific! */
      /* If available, use a timer interrupt to set DOSflagTimeOut */
      DOSflagTimeOut--;
   }
   RI1_CLR;                                        /* manually clear receive flag */
   /* DOS_UART_CTS = 1; */                         /* prevent DOSonCHIP from sending bytes to host */
   if (!DOSflagTimeOut) return DOS_HANDSHAKE_OFF;  /* if timeout, return no response */
   else return SBUF1;                              /* return UART byte */
}

/*******************************************************************************/
void DOSportFlushQ (void)                          /* Wait until transmit/receive queue is empty */
{
}

/*******************************************************************************/
void DOSportInit (void)                            /* Initialize host port */
{
   unsigned long int DOSflagTimeOut;

   TI1_SET;                                        /* set initial state for DOSportPutByte */

   /* DOS_UART_CTS = 0; */                         /* allow DOSonCHIP to send bytes to host */
   do
   {
      /* send at least 2 carriage returns for DOSonCHIP autobaud routine */
      DOSportPutByte (0x0D);                       /* send first carriage return */

      DOSflagTimeOut = 0x0004FFFF;                 /* wait a bit for DOSonCHIP to process */
      while (DOSflagTimeOut--);

      DOSportPutByte (0x0D);                       /* send confirming carriage return */

      DOSflagTimeOut = 0xFFFF;                     /* wait for byte to be received */
      while (!RI1 && DOSflagTimeOut)
      {
         /* Wait loop time is CPU specific! */
         /* If available, use a timer interrupt to set DOSflagTimeOut */
         DOSflagTimeOut--;
      }
      RI1_CLR;                                     /* manually clear receive flag */
   } while (SBUF1 != DOS_RES_NOERROR);
   /* CTS = 1; */                                  /* prevent DOSonCHIP from sending bytes to host */
}

#endif /* DOS_PORT_UART */
