
#include "SPI_FS.h" 
#include "SPI_FS_CS.h" 
#include "DOSport_SPI.h"                  
#include "DOSprotocol.h"                           /* DOSonCHIP communication protocol include file */

/******************************************************************************/
unsigned char DOSportGetNotBusy (void)             /* Poll DOSonCHIP for valid response byte */
{
   unsigned long int DOSflagTimeOut = 0xFFFFFFFF;
   unsigned char     response;

   /* return non DOS_HANDSHAKE_WAIT byte or DOS_HANDSHAKE_WAIT byte if timeout */
   while (((response = DOSportGetByte ()) == DOS_HANDSHAKE_WAIT) && DOSflagTimeOut)
   {
      /* Skip over fixed amount of DOS_HANDSHAKE_WAIT bytes */
      /* Wait loop time is CPU specific! */
      /* If available, use a timer interrupt to set DOSflagTimeOut */
      DOSflagTimeOut -= 1;
   }
   return (response);
}

/*******************************************************************************/
/* Transmit byte to DOSonCHIP */
unsigned char DOSportPutByte (unsigned char txByte) {
    byte recv = 0;

    SPI_FS_CS_PutVal(0);
    Cpu_Delay100US(1);
    
    
    /* SPI port simultaneously transmits and receives a byte */    
    SPI_FS_SendChar(txByte);
    SPI_FS_RecvChar(&recv);
    
    SPI_FS_CS_PutVal(1);
    return recv;
}

/*******************************************************************************/
unsigned char DOSportGetByte (void)                /* Receive byte from DOSonCHIP */
{  
   return (DOSportPutByte (DOS_HANDSHAKE_POLL));   /* transmit polling byte & return received byte */
}

/*******************************************************************************/
/* Wait until transmit/receive queue is empty */
void DOSportFlushQ (void) {
/*
    unsigned int n;
    while (1) {
        SPI_FS_CharsInRxBuf(&n);
        if (n == 0) {
            break;
        }
    }
*/
}

/*******************************************************************************/
/* Initialize host port */
void DOSportInit (void) {
    /* SPI port attributes:                */
    /* Data 8-bits sent/received MSB first */
    /* Clock polarity = 0                  */
    /* Clock phase = 0                     */
    
   unsigned char temp = DOS_HANDSHAKE_OFF;

   SPI_FS_Init();
   SPI_FS_CS_PutVal(1);    
   Cpu_Delay100US(1);
   
   /* poll until DOSonCHIP is ready to communicate */
   while (temp == DOS_HANDSHAKE_OFF) {
     temp = DOSportGetByte(); 
   }

   /* signal to DOSonCHIP that host is ready */
   DOSportPutByte(DOS_HANDSHAKE_GO);              
}
