/******************************************************************************
*
* BADGE BULLETIN BOARD SYSTEM
*
* Filename:		  BBBS.c
* Author:		  Mark Renouf (tweakt)
* Revision:		  1.0
* Last Updated:   Aug 2, 2009
*******************************************************************************/

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "PWMRed.h"
#include "PWMGreen.h"
#include "PWMBlue.h"

/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "strtol.h"

#include "BBBS.h"
#include "SPI_FS.h"
#include "SPI_FS_CS.h"
#include "LEDMuxTimer.h"
#include "DOSfileAPI.h"

/****************************************************************************
 ************************** Global variables ********************************
 ***************************************************************************/


/****************************************************************************
 ************************** Functions ***************************************
 ***************************************************************************/

void bbbs_init(void)
{ 
    byte cmd = (byte) 'V';
    
    // Bring SPI CS port low
    SPI_FS_CS_PutVal(0);
    Cpu_Delay100US(10); // 1ms
   
   	//Term1_SendStr("BBBS Initializing...");
   	
    Cpu_DisableInt(); // disable global interrupts 	
   	PWMRed_Enable();
   	PWMBlue_Enable();
   	PWMGreen_Enable();
	
    LEDMuxTimer_Enable();

	Cpu_EnableInt();  // re-enable global interrupts when we're done and ready for action

	Term1_CRLF();
	
}

int strcmp(const char *s1, const char *s2);

int strcmp(const char *s1, const char *s2) {
  while (*s1 == *s2++) {
      if (*s1++ == 0)
        return (0);
  }
  return (*(const unsigned char *)s1 - *(const unsigned char *)(s2 - 1)); 
}


char in[80];

void wait_for(const char *str) {
    //Term1_SendStr("Waiting for Caller...");
    for (;;) {
        while (!Term1_KeyPressed()) {
          Cpu_SetWaitMode();
        }
        Term1_ReadLine(in);

        //Term1_SendStr("Got: ");
        //Term1_SendStr(in);
        //Term1_CRLF();

        if (strcmp(str, (const char *)in) == 0) {
            return;
        }
    }
}

void bbbs_mainloop(void) {
    uint8_t ch;

    // WAIT FOR "RING"
    wait_for("RING");
    
    // SEND ATA
    Term1_SendStr("ATA"); Term1_CRLF();
    
    // WAIT FOR "CONNECT"
    wait_for("CONNECT");
    

    // BEGIN SESSION
    
    // IF "NO CARRIER" go back to top

    while (1) {
        Term1_CRLF();
        Term1_SendStr("BBBS> ");
        while (!Term1_KeyPressed()) {
          Cpu_SetWaitMode();
        }
        Term1_CRLF();
        
        Term1_ReadChar(&ch);
        Term1_CRLF();
        
        switch (ch) {
            case 'D':
            case 'd':
                fs_GetVersion();
                break;
            case 'R':
//            case 'r':
//                Term1_SendStr("Not yet implemented!");
//                Term1_CRLF();
//                break;
            case 'B':
            case 'b':
                blinky_lights();
                break;
            case 'h':
            case 'H':
            default:
                Term1_SendStr("B)linky Lights ");
                Term1_CRLF();
                Term1_SendStr("D)isk Test> ");
                Term1_CRLF();
                Term1_SendStr("R)eboot ");
                Term1_CRLF();
                break;
        }
    }
}

char s[32];

void blinky_lights() {
    int r, g, b;   

    // Yes, this is exploitable... :-(
    // Term ReadLine is crappy.
    //char *s = (char *) MemMgr_memCallocIM(1, 32);
    
    Term1_SendStr("Red [0-255] ? ");
    Term1_ReadLine(s);
    r = atoi(s);
        
    Term1_SendStr("Green [0-255] ? ");
    Term1_ReadLine(s);
    g = atoi(s);

    Term1_SendStr("Blue [0-255] ? ");
    Term1_ReadLine(s);
    b = atoi(s);
    
    
    //Term1_SendStr("Setting LED Color to: ");
    Term1_SendNum(r);
    Term1_SendStr(", ");
    Term1_SendNum(g);
    Term1_SendStr(", ");
    Term1_SendNum(b);
    Term1_CRLF();
    
    PWMRed_SetRatio8(~((byte) (r & 0xFF)));
    PWMGreen_SetRatio8(~((byte) (g & 0xFF)));
    PWMBlue_SetRatio8(~((byte) (b & 0xFF)));
}

led_state_type LEDstate = RED;

// TMR1: RGB multiplexing, every 1ms
void LEDMux(void) {
    switch (LEDstate) {
        case RED:
            setRegBit(GPIO_A_PER, PE0);
            clrRegBit(GPIO_A_PER, PE1);
            clrRegBit(GPIO_A_PER, PE2);
            LEDstate = GREEN;
        break;

        case GREEN:
            clrRegBit(GPIO_A_PER, PE0);
            setRegBit(GPIO_A_PER, PE1);
            clrRegBit(GPIO_A_PER, PE2);
            LEDstate = BLUE;					
        break;

        case BLUE:
            clrRegBit(GPIO_A_PER, PE0);
            clrRegBit(GPIO_A_PER, PE1);
            setRegBit(GPIO_A_PER, PE2);
            LEDstate = RED;				
        break;
    }
}

/*
 device_type 03(hex) : This byte indicates the device type is GOLDELOX-DOS
 silicon_rev This byte indicates the GOLDELOX silicon revision
 pmmc_rev This byte indicates the PmmC firmware revision
 reserved1 This byte is reserved for future support. If the value is 0 then ignore it
 reserved2 This byte is reserved for future support. If the value is 0 then ignore it

\Description This command requests all the necessary information from the device about its
            characteristics and capability.
*/
void fs_GetVersion() {
    unsigned char versionIC;
    unsigned int versionBoot;
    unsigned int versionFile;    
    

    DOSgetIC(&versionIC);
    DOSgetVersion(&versionBoot, &versionFile);

    Term1_CRLF();
    Term1_SendStr("DOSonCHIP Info");
    Term1_CRLF();
    
    Term1_SendStr("IC: ");
    Term1_SendNum(versionIC);
    Term1_CRLF();

    Term1_SendStr("Boot: ");
    Term1_SendNum(versionIC);
    Term1_CRLF();

    Term1_SendStr("File: ");
    Term1_SendNum(versionIC);
    Term1_CRLF();
}

